#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


    #include "kvec.h"
    #include "khash.h"
    #include "levenshtein.h"
    #include "medaka_bamiter.h"
    #include "medaka_common.h"
    #include "medaka_khcounter.h"
    #include "clair3_pileup.h"
    #include "clair3_full_alignment.h"
    

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 77), // _fa_data *()(char const *, char const *, char const *, Variant * *, size_t, size_t *, size_t, _Bool, size_t, size_t, size_t, size_t)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 107), // char const *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 49), // Variant * *
/*  5 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/*  6 */ _CFFI_OP(_CFFI_OP_POINTER, 5), // size_t *
/*  7 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/*  9 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 10 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 11 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 12 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 14 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // _plp_data *()(char const *, bam_fset const *, char const *, size_t, float, float, size_t, size_t, _Bool, size_t, _Bool)
/* 15 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 16 */ _CFFI_OP(_CFFI_OP_POINTER, 105), // bam_fset const *
/* 17 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 18 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 19 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 20 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 21 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 22 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 23 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1),
/* 24 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 25 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1),
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // _plp_data *()(size_t, size_t, size_t, size_t, size_t, size_t)
/* 28 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 29 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 30 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 31 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 32 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 33 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 34 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION, 84), // bam_fset *()(char *)
/* 36 */ _CFFI_OP(_CFFI_OP_POINTER, 107), // char *
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 38 */ _CFFI_OP(_CFFI_OP_FUNCTION, 36), // char *()(char *, size_t, size_t)
/* 39 */ _CFFI_OP(_CFFI_OP_NOOP, 36),
/* 40 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 41 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION, 36), // char *()(uint8_t *, size_t, size_t)
/* 44 */ _CFFI_OP(_CFFI_OP_POINTER, 119), // uint8_t *
/* 45 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 48 */ _CFFI_OP(_CFFI_OP_FUNCTION, 115), // int()(Variant *, Read *, size_t, size_t, size_t, char *, size_t)
/* 49 */ _CFFI_OP(_CFFI_OP_POINTER, 101), // Variant *
/* 50 */ _CFFI_OP(_CFFI_OP_POINTER, 100), // Read *
/* 51 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 52 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 53 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 54 */ _CFFI_OP(_CFFI_OP_NOOP, 36),
/* 55 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION, 115), // int()(Variants_info *, Read *, char *, size_t)
/* 58 */ _CFFI_OP(_CFFI_OP_POINTER, 102), // Variants_info *
/* 59 */ _CFFI_OP(_CFFI_OP_NOOP, 50),
/* 60 */ _CFFI_OP(_CFFI_OP_NOOP, 36),
/* 61 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 63 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // size_t()(size_t, size_t, size_t, size_t, size_t *)
/* 64 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 65 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 66 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 67 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 68 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 70 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // void()(HAP *, int *, size_t, size_t)
/* 71 */ _CFFI_OP(_CFFI_OP_POINTER, 97), // HAP *
/* 72 */ _CFFI_OP(_CFFI_OP_POINTER, 115), // int *
/* 73 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 74 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 76 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // void()(_fa_data *)
/* 77 */ _CFFI_OP(_CFFI_OP_POINTER, 103), // _fa_data *
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 79 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // void()(_plp_data *, _Bool)
/* 80 */ _CFFI_OP(_CFFI_OP_POINTER, 104), // _plp_data *
/* 81 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 83 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // void()(bam_fset *)
/* 84 */ _CFFI_OP(_CFFI_OP_POINTER, 105), // bam_fset *
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 86 */ _CFFI_OP(_CFFI_OP_FUNCTION, 120), // void()(uint32_t *, size_t, size_t *, size_t *, size_t, size_t, size_t, _Bool)
/* 87 */ _CFFI_OP(_CFFI_OP_POINTER, 118), // uint32_t *
/* 88 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 89 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 90 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 91 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 92 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 93 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 94 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1),
/* 95 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 96 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // Alt_info
/* 97 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // HAP
/* 98 */ _CFFI_OP(_CFFI_OP_POINTER, 99), // Pos_info *
/* 99 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // Pos_info
/* 100 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // Read
/* 101 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // Variant
/* 102 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // Variants_info
/* 103 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // _fa_data
/* 104 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // _plp_data
/* 105 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // bam_fset
/* 106 */ _CFFI_OP(_CFFI_OP_POINTER, 36), // char * *
/* 107 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 108 */ _CFFI_OP(_CFFI_OP_OPEN_ARRAY, 107), // char[]
/* 109 */ _CFFI_OP(_CFFI_OP_POINTER, 110), // int8_t *
/* 110 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 17), // int8_t
/* 111 */ _CFFI_OP(_CFFI_OP_ARRAY, 110), // int8_t[32]
/* 112 */ (_cffi_opcode_t)(32),
/* 113 */ _CFFI_OP(_CFFI_OP_ARRAY, 110), // int8_t[3]
/* 114 */ (_cffi_opcode_t)(3),
/* 115 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 116 */ _CFFI_OP(_CFFI_OP_ARRAY, 115), // int[32]
/* 117 */ (_cffi_opcode_t)(32),
/* 118 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22), // uint32_t
/* 119 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 18), // uint8_t
/* 120 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_bam_fset(bam_fset *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
}
struct _cffi_align_typedef_bam_fset { char x; bam_fset y; };

static _fa_data * _cffi_d_calculate_clair3_full_alignment(char const * x0, char const * x1, char const * x2, Variant * * x3, size_t x4, size_t * x5, size_t x6, _Bool x7, size_t x8, size_t x9, size_t x10, size_t x11)
{
  return calculate_clair3_full_alignment(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_calculate_clair3_full_alignment(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  char const * x2;
  Variant * * x3;
  size_t x4;
  size_t * x5;
  size_t x6;
  _Bool x7;
  size_t x8;
  size_t x9;
  size_t x10;
  size_t x11;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _fa_data * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;
  PyObject *arg10;
  PyObject *arg11;

  if (!PyArg_UnpackTuple(args, "calculate_clair3_full_alignment", 12, 12, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9, &arg10, &arg11))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (Variant * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, size_t);
  if (x4 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, size_t);
  if (x6 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x7 = (_Bool)_cffi_to_c__Bool(arg7);
  if (x7 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, size_t);
  if (x8 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x9 = _cffi_to_c_int(arg9, size_t);
  if (x9 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x10 = _cffi_to_c_int(arg10, size_t);
  if (x10 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x11 = _cffi_to_c_int(arg11, size_t);
  if (x11 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = calculate_clair3_full_alignment(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(77));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_calculate_clair3_full_alignment _cffi_d_calculate_clair3_full_alignment
#endif

static _plp_data * _cffi_d_calculate_clair3_pileup(char const * x0, bam_fset const * x1, char const * x2, size_t x3, float x4, float x5, size_t x6, size_t x7, _Bool x8, size_t x9, _Bool x10)
{
  return calculate_clair3_pileup(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_calculate_clair3_pileup(PyObject *self, PyObject *args)
{
  char const * x0;
  bam_fset const * x1;
  char const * x2;
  size_t x3;
  float x4;
  float x5;
  size_t x6;
  size_t x7;
  _Bool x8;
  size_t x9;
  _Bool x10;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _plp_data * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;
  PyObject *arg10;

  if (!PyArg_UnpackTuple(args, "calculate_clair3_pileup", 11, 11, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9, &arg10))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(16), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (bam_fset const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(16), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  x5 = (float)_cffi_to_c_float(arg5);
  if (x5 == (float)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, size_t);
  if (x6 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x7 = _cffi_to_c_int(arg7, size_t);
  if (x7 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x8 = (_Bool)_cffi_to_c__Bool(arg8);
  if (x8 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  x9 = _cffi_to_c_int(arg9, size_t);
  if (x9 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x10 = (_Bool)_cffi_to_c__Bool(arg10);
  if (x10 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = calculate_clair3_pileup(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(80));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_calculate_clair3_pileup _cffi_d_calculate_clair3_pileup
#endif

static void _cffi_d_cigar_prefix_length(uint32_t * x0, size_t x1, size_t * x2, size_t * x3, size_t x4, size_t x5, size_t x6, _Bool x7)
{
  cigar_prefix_length(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_cigar_prefix_length(PyObject *self, PyObject *args)
{
  uint32_t * x0;
  size_t x1;
  size_t * x2;
  size_t * x3;
  size_t x4;
  size_t x5;
  size_t x6;
  _Bool x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "cigar_prefix_length", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(87), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(87), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, size_t);
  if (x4 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, size_t);
  if (x5 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x6 = _cffi_to_c_int(arg6, size_t);
  if (x6 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x7 = (_Bool)_cffi_to_c__Bool(arg7);
  if (x7 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { cigar_prefix_length(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_cigar_prefix_length _cffi_d_cigar_prefix_length
#endif

static bam_fset * _cffi_d_create_bam_fset(char * x0)
{
  return create_bam_fset(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_create_bam_fset(PyObject *self, PyObject *arg0)
{
  char * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  bam_fset * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(36), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(36), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = create_bam_fset(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(84));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_create_bam_fset _cffi_d_create_bam_fset
#endif

static _plp_data * _cffi_d_create_plp_data(size_t x0, size_t x1, size_t x2, size_t x3, size_t x4, size_t x5)
{
  return create_plp_data(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_create_plp_data(PyObject *self, PyObject *args)
{
  size_t x0;
  size_t x1;
  size_t x2;
  size_t x3;
  size_t x4;
  size_t x5;
  _plp_data * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "create_plp_data", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, size_t);
  if (x0 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, size_t);
  if (x4 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, size_t);
  if (x5 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = create_plp_data(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(80));
  return pyresult;
}
#else
#  define _cffi_f_create_plp_data _cffi_d_create_plp_data
#endif

static void _cffi_d_destroy_bam_fset(bam_fset * x0)
{
  destroy_bam_fset(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_destroy_bam_fset(PyObject *self, PyObject *arg0)
{
  bam_fset * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(84), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (bam_fset *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(84), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { destroy_bam_fset(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_destroy_bam_fset _cffi_d_destroy_bam_fset
#endif

static void _cffi_d_destroy_fa_data(_fa_data * x0)
{
  destroy_fa_data(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_destroy_fa_data(PyObject *self, PyObject *arg0)
{
  _fa_data * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(77), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (_fa_data *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(77), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { destroy_fa_data(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_destroy_fa_data _cffi_d_destroy_fa_data
#endif

static void _cffi_d_destroy_plp_data(_plp_data * x0, _Bool x1)
{
  destroy_plp_data(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_destroy_plp_data(PyObject *self, PyObject *args)
{
  _plp_data * x0;
  _Bool x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "destroy_plp_data", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(80), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (_plp_data *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(80), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (_Bool)_cffi_to_c__Bool(arg1);
  if (x1 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { destroy_plp_data(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_destroy_plp_data _cffi_d_destroy_plp_data
#endif

static size_t _cffi_d_get_overlap_candidate_num(size_t x0, size_t x1, size_t x2, size_t x3, size_t * x4)
{
  return get_overlap_candidate_num(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_get_overlap_candidate_num(PyObject *self, PyObject *args)
{
  size_t x0;
  size_t x1;
  size_t x2;
  size_t x3;
  size_t * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  size_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "get_overlap_candidate_num", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, size_t);
  if (x0 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(6), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(6), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = get_overlap_candidate_num(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_get_overlap_candidate_num _cffi_d_get_overlap_candidate_num
#endif

static char * _cffi_d_get_query_seq(uint8_t * x0, size_t x1, size_t x2)
{
  return get_query_seq(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_get_query_seq(PyObject *self, PyObject *args)
{
  uint8_t * x0;
  size_t x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "get_query_seq", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(44), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (uint8_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(44), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = get_query_seq(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(36));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_get_query_seq _cffi_d_get_query_seq
#endif

static char * _cffi_d_get_ref_seq(char * x0, size_t x1, size_t x2)
{
  return get_ref_seq(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_get_ref_seq(PyObject *self, PyObject *args)
{
  char * x0;
  size_t x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "get_ref_seq", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(36), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(36), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = get_ref_seq(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(36));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_get_ref_seq _cffi_d_get_ref_seq
#endif

static int _cffi_d_haplotag_read(Variants_info * x0, Read * x1, char * x2, size_t x3)
{
  return haplotag_read(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_haplotag_read(PyObject *self, PyObject *args)
{
  Variants_info * x0;
  Read * x1;
  char * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "haplotag_read", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(58), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Variants_info *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(58), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(50), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (Read *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(50), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(36), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(36), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = haplotag_read(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_haplotag_read _cffi_d_haplotag_read
#endif

static int _cffi_d_realign_read(Variant * x0, Read * x1, size_t x2, size_t x3, size_t x4, char * x5, size_t x6)
{
  return realign_read(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_realign_read(PyObject *self, PyObject *args)
{
  Variant * x0;
  Read * x1;
  size_t x2;
  size_t x3;
  size_t x4;
  char * x5;
  size_t x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "realign_read", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(49), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Variant *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(49), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(50), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (Read *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(50), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, size_t);
  if (x4 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(36), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(36), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, size_t);
  if (x6 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = realign_read(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_realign_read _cffi_d_realign_read
#endif

static void _cffi_d_sort_read_name_by_haplotype(HAP * x0, int * x1, size_t x2, size_t x3)
{
  sort_read_name_by_haplotype(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sort_read_name_by_haplotype(PyObject *self, PyObject *args)
{
  HAP * x0;
  int * x1;
  size_t x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "sort_read_name_by_haplotype", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(71), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (HAP *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(71), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sort_read_name_by_haplotype(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sort_read_name_by_haplotype _cffi_d_sort_read_name_by_haplotype
#endif

static int _cffi_const_MAX_INDEL_LENGTH(unsigned long long *o)
{
  int n = (MAX_INDEL_LENGTH) <= 0;
  *o = (unsigned long long)((MAX_INDEL_LENGTH) | 0);  /* check that MAX_INDEL_LENGTH is an integer */
  if (!_cffi_check_int(*o, n, 50U))
    n |= 2;
  return n;
}

static int _cffi_const_MAX_READ_COUNT(unsigned long long *o)
{
  int n = (MAX_READ_COUNT) <= 0;
  *o = (unsigned long long)((MAX_READ_COUNT) | 0);  /* check that MAX_READ_COUNT is an integer */
  if (!_cffi_check_int(*o, n, 1000U))
    n |= 2;
  return n;
}

static int _cffi_const_SAMTOOLS_VIEW_FILTER_FLAG(unsigned long long *o)
{
  int n = (SAMTOOLS_VIEW_FILTER_FLAG) <= 0;
  *o = (unsigned long long)((SAMTOOLS_VIEW_FILTER_FLAG) | 0);  /* check that SAMTOOLS_VIEW_FILTER_FLAG is an integer */
  if (!_cffi_check_int(*o, n, 2316U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_fwd_del(unsigned long long *o)
{
  int n = (c3_fwd_del) <= 0;
  *o = (unsigned long long)((c3_fwd_del) | 0);  /* check that c3_fwd_del is an integer */
  if (!_cffi_check_int(*o, n, 8U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_fwd_del_all(unsigned long long *o)
{
  int n = (c3_fwd_del_all) <= 0;
  *o = (unsigned long long)((c3_fwd_del_all) | 0);  /* check that c3_fwd_del_all is an integer */
  if (!_cffi_check_int(*o, n, 6U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_fwd_del_best(unsigned long long *o)
{
  int n = (c3_fwd_del_best) <= 0;
  *o = (unsigned long long)((c3_fwd_del_best) | 0);  /* check that c3_fwd_del_best is an integer */
  if (!_cffi_check_int(*o, n, 7U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_fwd_ins_all(unsigned long long *o)
{
  int n = (c3_fwd_ins_all) <= 0;
  *o = (unsigned long long)((c3_fwd_ins_all) | 0);  /* check that c3_fwd_ins_all is an integer */
  if (!_cffi_check_int(*o, n, 4U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_fwd_ins_best(unsigned long long *o)
{
  int n = (c3_fwd_ins_best) <= 0;
  *o = (unsigned long long)((c3_fwd_ins_best) | 0);  /* check that c3_fwd_ins_best is an integer */
  if (!_cffi_check_int(*o, n, 5U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_rev_del(unsigned long long *o)
{
  int n = (c3_rev_del) <= 0;
  *o = (unsigned long long)((c3_rev_del) | 0);  /* check that c3_rev_del is an integer */
  if (!_cffi_check_int(*o, n, 17U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_rev_del_all(unsigned long long *o)
{
  int n = (c3_rev_del_all) <= 0;
  *o = (unsigned long long)((c3_rev_del_all) | 0);  /* check that c3_rev_del_all is an integer */
  if (!_cffi_check_int(*o, n, 15U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_rev_del_best(unsigned long long *o)
{
  int n = (c3_rev_del_best) <= 0;
  *o = (unsigned long long)((c3_rev_del_best) | 0);  /* check that c3_rev_del_best is an integer */
  if (!_cffi_check_int(*o, n, 16U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_rev_ins_all(unsigned long long *o)
{
  int n = (c3_rev_ins_all) <= 0;
  *o = (unsigned long long)((c3_rev_ins_all) | 0);  /* check that c3_rev_ins_all is an integer */
  if (!_cffi_check_int(*o, n, 13U))
    n |= 2;
  return n;
}

static int _cffi_const_c3_rev_ins_best(unsigned long long *o)
{
  int n = (c3_rev_ins_best) <= 0;
  *o = (unsigned long long)((c3_rev_ins_best) | 0);  /* check that c3_rev_ins_best is an integer */
  if (!_cffi_check_int(*o, n, 14U))
    n |= 2;
  return n;
}

static int _cffi_const_channel_size(unsigned long long *o)
{
  int n = (channel_size) <= 0;
  *o = (unsigned long long)((channel_size) | 0);  /* check that channel_size is an integer */
  if (!_cffi_check_int(*o, n, 8U))
    n |= 2;
  return n;
}

static int _cffi_const_expand_reference_region(unsigned long long *o)
{
  int n = (expand_reference_region) <= 0;
  *o = (unsigned long long)((expand_reference_region) | 0);  /* check that expand_reference_region is an integer */
  if (!_cffi_check_int(*o, n, 2000000U))
    n |= 2;
  return n;
}

static int _cffi_const_featlenclair3(unsigned long long *o)
{
  int n = (featlenclair3) <= 0;
  *o = (unsigned long long)((featlenclair3) | 0);  /* check that featlenclair3 is an integer */
  if (!_cffi_check_int(*o, n, 18U))
    n |= 2;
  return n;
}

static int _cffi_const_flanking_base_num(unsigned long long *o)
{
  int n = (flanking_base_num) <= 0;
  *o = (unsigned long long)((flanking_base_num) | 0);  /* check that flanking_base_num is an integer */
  if (!_cffi_check_int(*o, n, 16U))
    n |= 2;
  return n;
}

static int _cffi_const_min_bq(unsigned long long *o)
{
  int n = (min_bq) <= 0;
  *o = (unsigned long long)((min_bq) | 0);  /* check that min_bq is an integer */
  if (!_cffi_check_int(*o, n, 0))
    n |= 2;
  return n;
}

static int _cffi_const_min_haplotag_mq(unsigned long long *o)
{
  int n = (min_haplotag_mq) <= 0;
  *o = (unsigned long long)((min_haplotag_mq) | 0);  /* check that min_haplotag_mq is an integer */
  if (!_cffi_check_int(*o, n, 20U))
    n |= 2;
  return n;
}

static int _cffi_const_mpileup_expand_reference_region(unsigned long long *o)
{
  int n = (mpileup_expand_reference_region) <= 0;
  *o = (unsigned long long)((mpileup_expand_reference_region) | 0);  /* check that mpileup_expand_reference_region is an integer */
  if (!_cffi_check_int(*o, n, 1000U))
    n |= 2;
  return n;
}

static int _cffi_const_no_of_positions(unsigned long long *o)
{
  int n = (no_of_positions) <= 0;
  *o = (unsigned long long)((no_of_positions) | 0);  /* check that no_of_positions is an integer */
  if (!_cffi_check_int(*o, n, 33U))
    n |= 2;
  return n;
}

static int _cffi_const_overhang(unsigned long long *o)
{
  int n = (overhang) <= 0;
  *o = (unsigned long long)((overhang) | 0);  /* check that overhang is an integer */
  if (!_cffi_check_int(*o, n, 10U))
    n |= 2;
  return n;
}

static int _cffi_const_pileup_flanking_base_num(unsigned long long *o)
{
  int n = (pileup_flanking_base_num) <= 0;
  *o = (unsigned long long)((pileup_flanking_base_num) | 0);  /* check that pileup_flanking_base_num is an integer */
  if (!_cffi_check_int(*o, n, 16U))
    n |= 2;
  return n;
}

static int _cffi_const_reverse_pos_start(unsigned long long *o)
{
  int n = (reverse_pos_start) <= 0;
  *o = (unsigned long long)((reverse_pos_start) | 0);  /* check that reverse_pos_start is an integer */
  if (!_cffi_check_int(*o, n, 9U))
    n |= 2;
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld__Alt_info(Alt_info *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->ins_bases; (void)tmp; }
  { char *tmp = &p->alt_base; (void)tmp; }
  (void)((p->del_length) | 0);  /* check that 'Alt_info.del_length' is an integer */
  (void)((p->has_alt_info) | 0);  /* check that 'Alt_info.has_alt_info' is an integer */
}
struct _cffi_align__Alt_info { char x; Alt_info y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__HAP(HAP *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->read_index) | 0);  /* check that 'HAP.read_index' is an integer */
  (void)((p->haplotype) | 0);  /* check that 'HAP.haplotype' is an integer */
}
struct _cffi_align__HAP { char x; HAP y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Pos_info(Pos_info *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->ins_bases; (void)tmp; }
  (void)((p->ins_length) | 0);  /* check that 'Pos_info.ins_length' is an integer */
  (void)((p->alt_base) | 0);  /* check that 'Pos_info.alt_base' is an integer */
  (void)((p->del_length) | 0);  /* check that 'Pos_info.del_length' is an integer */
  (void)((p->bq) | 0);  /* check that 'Pos_info.bq' is an integer */
}
struct _cffi_align__Pos_info { char x; Pos_info y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Read(Read *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->read_start) | 0);  /* check that 'Read.read_start' is an integer */
  { char * *tmp = &p->q_name; (void)tmp; }
  (void)((p->read_end) | 0);  /* check that 'Read.read_end' is an integer */
  { uint32_t * *tmp = &p->cigartuples; (void)tmp; }
  { uint8_t * *tmp = &p->seqi; (void)tmp; }
  { uint8_t * *tmp = &p->qual; (void)tmp; }
  (void)((p->mq) | 0);  /* check that 'Read.mq' is an integer */
  (void)((p->n_cigar) | 0);  /* check that 'Read.n_cigar' is an integer */
  (void)((p->l_qseq) | 0);  /* check that 'Read.l_qseq' is an integer */
  (void)((p->haplotype) | 0);  /* check that 'Read.haplotype' is an integer */
  (void)((p->strand) | 0);  /* check that 'Read.strand' is an integer */
  { Pos_info * *tmp = &p->pos_info; (void)tmp; }
  (void)((p->overlap_candidates_num) | 0);  /* check that 'Read.overlap_candidates_num' is an integer */
  (void)((p->flanking_start) | 0);  /* check that 'Read.flanking_start' is an integer */
}
struct _cffi_align__Read { char x; Read y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Variant(Variant *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->position) | 0);  /* check that 'Variant.position' is an integer */
  { char *tmp = &p->ref_base; (void)tmp; }
  { char *tmp = &p->alt_base; (void)tmp; }
  (void)((p->genotype) | 0);  /* check that 'Variant.genotype' is an integer */
  (void)((p->phase_set) | 0);  /* check that 'Variant.phase_set' is an integer */
}
struct _cffi_align__Variant { char x; Variant y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Variants_info(Variants_info *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Variant * * *tmp = &p->variants; (void)tmp; }
  (void)((p->variant_num) | 0);  /* check that 'Variants_info.variant_num' is an integer */
  (void)((p->variant_current_pos) | 0);  /* check that 'Variants_info.variant_current_pos' is an integer */
}
struct _cffi_align__Variants_info { char x; Variants_info y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld___fa_data(_fa_data *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int8_t * *tmp = &p->matrix; (void)tmp; }
  { char * * *tmp = &p->all_alt_info; (void)tmp; }
  (void)((p->candidates_num) | 0);  /* check that '_fa_data.candidates_num' is an integer */
}
struct _cffi_align___fa_data { char x; _fa_data y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld___plp_data(_plp_data *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->buffer_cols) | 0);  /* check that '_plp_data.buffer_cols' is an integer */
  (void)((p->num_dtypes) | 0);  /* check that '_plp_data.num_dtypes' is an integer */
  (void)((p->num_homop) | 0);  /* check that '_plp_data.num_homop' is an integer */
  (void)((p->n_cols) | 0);  /* check that '_plp_data.n_cols' is an integer */
  { size_t * *tmp = &p->matrix; (void)tmp; }
  { size_t * *tmp = &p->major; (void)tmp; }
  { size_t * *tmp = &p->minor; (void)tmp; }
  { char * * *tmp = &p->all_alt_info; (void)tmp; }
  (void)((p->candidates_num) | 0);  /* check that '_plp_data.candidates_num' is an integer */
  { size_t * *tmp = &p->pos_ref_count; (void)tmp; }
  { size_t * *tmp = &p->pos_total_count; (void)tmp; }
}
struct _cffi_align___plp_data { char x; _plp_data y; };

static char const *_cffi_var_ACGT(void)
{
  return (ACGT);
}

static int8_t const (*_cffi_var_HAP_TYPE(void))[3]
{
  return &(HAP_TYPE);
}

static char const * *_cffi_var_RN(void)
{
  return &(RN);
}

static int8_t const (*_cffi_var_acgt2num(void))[32]
{
  return &(acgt2num);
}

static int const (*_cffi_var_base2index(void))[32]
{
  return &(base2index);
}

static int const (*_cffi_var_num2countbase(void))[32]
{
  return &(num2countbase);
}

static int8_t const (*_cffi_var_num2countbase_fa(void))[32]
{
  return &(num2countbase_fa);
}

static int const (*_cffi_var_num2countbaseclair3(void))[32]
{
  return &(num2countbaseclair3);
}

static char const *_cffi_var_plp_bases_clair3(void)
{
  return (plp_bases_clair3);
}

static const struct _cffi_global_s _cffi_globals[] = {
  { "ACGT", (void *)_cffi_var_ACGT, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 108), (void *)0 },
  { "HAP_TYPE", (void *)_cffi_var_HAP_TYPE, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 113), (void *)0 },
  { "MAX_INDEL_LENGTH", (void *)_cffi_const_MAX_INDEL_LENGTH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "MAX_READ_COUNT", (void *)_cffi_const_MAX_READ_COUNT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "RN", (void *)_cffi_var_RN, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 1), (void *)0 },
  { "SAMTOOLS_VIEW_FILTER_FLAG", (void *)_cffi_const_SAMTOOLS_VIEW_FILTER_FLAG, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "acgt2num", (void *)_cffi_var_acgt2num, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 111), (void *)0 },
  { "base2index", (void *)_cffi_var_base2index, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 116), (void *)0 },
  { "c3_fwd_del", (void *)_cffi_const_c3_fwd_del, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_fwd_del_all", (void *)_cffi_const_c3_fwd_del_all, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_fwd_del_best", (void *)_cffi_const_c3_fwd_del_best, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_fwd_ins_all", (void *)_cffi_const_c3_fwd_ins_all, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_fwd_ins_best", (void *)_cffi_const_c3_fwd_ins_best, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_rev_del", (void *)_cffi_const_c3_rev_del, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_rev_del_all", (void *)_cffi_const_c3_rev_del_all, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_rev_del_best", (void *)_cffi_const_c3_rev_del_best, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_rev_ins_all", (void *)_cffi_const_c3_rev_ins_all, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "c3_rev_ins_best", (void *)_cffi_const_c3_rev_ins_best, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "calculate_clair3_full_alignment", (void *)_cffi_f_calculate_clair3_full_alignment, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_calculate_clair3_full_alignment },
  { "calculate_clair3_pileup", (void *)_cffi_f_calculate_clair3_pileup, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 14), (void *)_cffi_d_calculate_clair3_pileup },
  { "channel_size", (void *)_cffi_const_channel_size, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "cigar_prefix_length", (void *)_cffi_f_cigar_prefix_length, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 86), (void *)_cffi_d_cigar_prefix_length },
  { "create_bam_fset", (void *)_cffi_f_create_bam_fset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 35), (void *)_cffi_d_create_bam_fset },
  { "create_plp_data", (void *)_cffi_f_create_plp_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_create_plp_data },
  { "destroy_bam_fset", (void *)_cffi_f_destroy_bam_fset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 83), (void *)_cffi_d_destroy_bam_fset },
  { "destroy_fa_data", (void *)_cffi_f_destroy_fa_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 76), (void *)_cffi_d_destroy_fa_data },
  { "destroy_plp_data", (void *)_cffi_f_destroy_plp_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 79), (void *)_cffi_d_destroy_plp_data },
  { "expand_reference_region", (void *)_cffi_const_expand_reference_region, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "featlenclair3", (void *)_cffi_const_featlenclair3, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "flanking_base_num", (void *)_cffi_const_flanking_base_num, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "get_overlap_candidate_num", (void *)_cffi_f_get_overlap_candidate_num, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 63), (void *)_cffi_d_get_overlap_candidate_num },
  { "get_query_seq", (void *)_cffi_f_get_query_seq, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 43), (void *)_cffi_d_get_query_seq },
  { "get_ref_seq", (void *)_cffi_f_get_ref_seq, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 38), (void *)_cffi_d_get_ref_seq },
  { "haplotag_read", (void *)_cffi_f_haplotag_read, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 57), (void *)_cffi_d_haplotag_read },
  { "min_bq", (void *)_cffi_const_min_bq, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "min_haplotag_mq", (void *)_cffi_const_min_haplotag_mq, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "mpileup_expand_reference_region", (void *)_cffi_const_mpileup_expand_reference_region, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "no_of_positions", (void *)_cffi_const_no_of_positions, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "num2countbase", (void *)_cffi_var_num2countbase, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 116), (void *)0 },
  { "num2countbase_fa", (void *)_cffi_var_num2countbase_fa, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 111), (void *)0 },
  { "num2countbaseclair3", (void *)_cffi_var_num2countbaseclair3, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 116), (void *)0 },
  { "overhang", (void *)_cffi_const_overhang, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "pileup_flanking_base_num", (void *)_cffi_const_pileup_flanking_base_num, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "plp_bases_clair3", (void *)_cffi_var_plp_bases_clair3, _CFFI_OP(_CFFI_OP_GLOBAL_VAR_F, 108), (void *)0 },
  { "realign_read", (void *)_cffi_f_realign_read, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 48), (void *)_cffi_d_realign_read },
  { "reverse_pos_start", (void *)_cffi_const_reverse_pos_start, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "sort_read_name_by_haplotype", (void *)_cffi_f_sort_read_name_by_haplotype, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 70), (void *)_cffi_d_sort_read_name_by_haplotype },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "ins_bases", offsetof(Alt_info, ins_bases),
                 sizeof(((Alt_info *)0)->ins_bases),
                 _CFFI_OP(_CFFI_OP_NOOP, 36) },
  { "alt_base", offsetof(Alt_info, alt_base),
                sizeof(((Alt_info *)0)->alt_base),
                _CFFI_OP(_CFFI_OP_NOOP, 107) },
  { "del_length", offsetof(Alt_info, del_length),
                  sizeof(((Alt_info *)0)->del_length),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "has_alt_info", offsetof(Alt_info, has_alt_info),
                    sizeof(((Alt_info *)0)->has_alt_info),
                    _CFFI_OP(_CFFI_OP_NOOP, 8) },
  { "read_index", offsetof(HAP, read_index),
                  sizeof(((HAP *)0)->read_index),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "haplotype", offsetof(HAP, haplotype),
                 sizeof(((HAP *)0)->haplotype),
                 _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "ins_bases", offsetof(Pos_info, ins_bases),
                 sizeof(((Pos_info *)0)->ins_bases),
                 _CFFI_OP(_CFFI_OP_NOOP, 36) },
  { "ins_length", offsetof(Pos_info, ins_length),
                  sizeof(((Pos_info *)0)->ins_length),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "alt_base", offsetof(Pos_info, alt_base),
                sizeof(((Pos_info *)0)->alt_base),
                _CFFI_OP(_CFFI_OP_NOOP, 115) },
  { "del_length", offsetof(Pos_info, del_length),
                  sizeof(((Pos_info *)0)->del_length),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "bq", offsetof(Pos_info, bq),
          sizeof(((Pos_info *)0)->bq),
          _CFFI_OP(_CFFI_OP_NOOP, 110) },
  { "read_start", offsetof(Read, read_start),
                  sizeof(((Read *)0)->read_start),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "q_name", offsetof(Read, q_name),
              sizeof(((Read *)0)->q_name),
              _CFFI_OP(_CFFI_OP_NOOP, 36) },
  { "read_end", offsetof(Read, read_end),
                sizeof(((Read *)0)->read_end),
                _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "cigartuples", offsetof(Read, cigartuples),
                   sizeof(((Read *)0)->cigartuples),
                   _CFFI_OP(_CFFI_OP_NOOP, 87) },
  { "seqi", offsetof(Read, seqi),
            sizeof(((Read *)0)->seqi),
            _CFFI_OP(_CFFI_OP_NOOP, 44) },
  { "qual", offsetof(Read, qual),
            sizeof(((Read *)0)->qual),
            _CFFI_OP(_CFFI_OP_NOOP, 44) },
  { "mq", offsetof(Read, mq),
          sizeof(((Read *)0)->mq),
          _CFFI_OP(_CFFI_OP_NOOP, 110) },
  { "n_cigar", offsetof(Read, n_cigar),
               sizeof(((Read *)0)->n_cigar),
               _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "l_qseq", offsetof(Read, l_qseq),
              sizeof(((Read *)0)->l_qseq),
              _CFFI_OP(_CFFI_OP_NOOP, 118) },
  { "haplotype", offsetof(Read, haplotype),
                 sizeof(((Read *)0)->haplotype),
                 _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "strand", offsetof(Read, strand),
              sizeof(((Read *)0)->strand),
              _CFFI_OP(_CFFI_OP_NOOP, 110) },
  { "pos_info", offsetof(Read, pos_info),
                sizeof(((Read *)0)->pos_info),
                _CFFI_OP(_CFFI_OP_NOOP, 98) },
  { "overlap_candidates_num", offsetof(Read, overlap_candidates_num),
                              sizeof(((Read *)0)->overlap_candidates_num),
                              _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "flanking_start", offsetof(Read, flanking_start),
                      sizeof(((Read *)0)->flanking_start),
                      _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "position", offsetof(Variant, position),
                sizeof(((Variant *)0)->position),
                _CFFI_OP(_CFFI_OP_NOOP, 115) },
  { "ref_base", offsetof(Variant, ref_base),
                sizeof(((Variant *)0)->ref_base),
                _CFFI_OP(_CFFI_OP_NOOP, 107) },
  { "alt_base", offsetof(Variant, alt_base),
                sizeof(((Variant *)0)->alt_base),
                _CFFI_OP(_CFFI_OP_NOOP, 107) },
  { "genotype", offsetof(Variant, genotype),
                sizeof(((Variant *)0)->genotype),
                _CFFI_OP(_CFFI_OP_NOOP, 115) },
  { "phase_set", offsetof(Variant, phase_set),
                 sizeof(((Variant *)0)->phase_set),
                 _CFFI_OP(_CFFI_OP_NOOP, 115) },
  { "variants", offsetof(Variants_info, variants),
                sizeof(((Variants_info *)0)->variants),
                _CFFI_OP(_CFFI_OP_NOOP, 4) },
  { "variant_num", offsetof(Variants_info, variant_num),
                   sizeof(((Variants_info *)0)->variant_num),
                   _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "variant_current_pos", offsetof(Variants_info, variant_current_pos),
                           sizeof(((Variants_info *)0)->variant_current_pos),
                           _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "matrix", offsetof(_fa_data, matrix),
              sizeof(((_fa_data *)0)->matrix),
              _CFFI_OP(_CFFI_OP_NOOP, 109) },
  { "all_alt_info", offsetof(_fa_data, all_alt_info),
                    sizeof(((_fa_data *)0)->all_alt_info),
                    _CFFI_OP(_CFFI_OP_NOOP, 106) },
  { "candidates_num", offsetof(_fa_data, candidates_num),
                      sizeof(((_fa_data *)0)->candidates_num),
                      _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "buffer_cols", offsetof(_plp_data, buffer_cols),
                   sizeof(((_plp_data *)0)->buffer_cols),
                   _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "num_dtypes", offsetof(_plp_data, num_dtypes),
                  sizeof(((_plp_data *)0)->num_dtypes),
                  _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "num_homop", offsetof(_plp_data, num_homop),
                 sizeof(((_plp_data *)0)->num_homop),
                 _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "n_cols", offsetof(_plp_data, n_cols),
              sizeof(((_plp_data *)0)->n_cols),
              _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "matrix", offsetof(_plp_data, matrix),
              sizeof(((_plp_data *)0)->matrix),
              _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "major", offsetof(_plp_data, major),
             sizeof(((_plp_data *)0)->major),
             _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "minor", offsetof(_plp_data, minor),
             sizeof(((_plp_data *)0)->minor),
             _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "all_alt_info", offsetof(_plp_data, all_alt_info),
                    sizeof(((_plp_data *)0)->all_alt_info),
                    _CFFI_OP(_CFFI_OP_NOOP, 106) },
  { "candidates_num", offsetof(_plp_data, candidates_num),
                      sizeof(((_plp_data *)0)->candidates_num),
                      _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "pos_ref_count", offsetof(_plp_data, pos_ref_count),
                     sizeof(((_plp_data *)0)->pos_ref_count),
                     _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "pos_total_count", offsetof(_plp_data, pos_total_count),
                       sizeof(((_plp_data *)0)->pos_total_count),
                       _CFFI_OP(_CFFI_OP_NOOP, 6) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$bam_fset", 105, 0,
    sizeof(bam_fset), offsetof(struct _cffi_align_typedef_bam_fset, y), 0, 0 },
  { "Alt_info", 96, _CFFI_F_CHECK_FIELDS,
    sizeof(Alt_info), offsetof(struct _cffi_align__Alt_info, y), 0, 4 },
  { "HAP", 97, _CFFI_F_CHECK_FIELDS,
    sizeof(HAP), offsetof(struct _cffi_align__HAP, y), 4, 2 },
  { "Pos_info", 99, _CFFI_F_CHECK_FIELDS,
    sizeof(Pos_info), offsetof(struct _cffi_align__Pos_info, y), 6, 5 },
  { "Read", 100, _CFFI_F_CHECK_FIELDS,
    sizeof(Read), offsetof(struct _cffi_align__Read, y), 11, 14 },
  { "Variant", 101, _CFFI_F_CHECK_FIELDS,
    sizeof(Variant), offsetof(struct _cffi_align__Variant, y), 25, 5 },
  { "Variants_info", 102, _CFFI_F_CHECK_FIELDS,
    sizeof(Variants_info), offsetof(struct _cffi_align__Variants_info, y), 30, 3 },
  { "_fa_data", 103, _CFFI_F_CHECK_FIELDS,
    sizeof(_fa_data), offsetof(struct _cffi_align___fa_data, y), 33, 3 },
  { "_plp_data", 104, _CFFI_F_CHECK_FIELDS,
    sizeof(_plp_data), offsetof(struct _cffi_align___plp_data, y), 36, 11 },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "Alt_info", 96 },
  { "HAP", 97 },
  { "Pos_info", 99 },
  { "Read", 100 },
  { "Variant", 101 },
  { "Variants_info", 102 },
  { "_fa_data", 103 },
  { "_plp_data", 104 },
  { "bam_fset", 105 },
  { "fa_data", 77 },
  { "plp_data", 80 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  NULL,  /* no enums */
  _cffi_typenames,
  47,  /* num_globals */
  9,  /* num_struct_unions */
  0,  /* num_enums */
  11,  /* num_typenames */
  NULL,  /* no includes */
  121,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_libclair3(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_libclair3(void) { return NULL; }
#  else
     initlibclair3(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_libclair3(void)
{
  return _cffi_init("libclair3", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
initlibclair3(void)
{
  _cffi_init("libclair3", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
